/*
 * Copyright (c) 2014-2019 Meltytech, LLC
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
import QtQuick          2.1
import QtQuick.Controls 1.0
import QtQuick.Layouts  1.0
import Shotcut.Controls 1.0
import QtQuick.Dialogs  1.0
//ZZimport org.shotcut.qml  1.0 as Shotcut
//import QtQuick.Controls.Styles 1.1

Item {
	
	property string filterName : 'JRtransition'
	
    width : 350
    height: 180

/* Javascript defaults
	params.framerate     = 25.0;
	params.delay         = 0.0;
	params.duration      = 4.0;
	params.angle         = 0;
	params.direction     = 3;
	params.main_color    = '#add8e6';
	params.stripe_color  = '#5f4f97';
	params.chroma        = false;
	params.reverse       = false;
	params.mask          = false;
	params.chroma_color  = '#00ff00';
*/
	
	property string fp0  : 'fp0'
	property string fp1  : 'fp1'
	property string fp2  : 'fp2'
	property string fp3  : 'fp3'
	property string fp4  : 'fp4'
	property string fp5  : 'fp5'
	property string fp6  : 'fp6'
	property string fp7  : 'fp7'
	property string fp8  : 'fp8'
	property string fp9  : 'fp9'
	property string fp10 : 'fp10'
	property string fp11 : 'fp11'

	Item {
		id:	fpid1										// Formal Parameter
		property string ptext		: 'Framerate'		// Formal Parameter title shown on filter
		property real   dflt		: 25.0				// Default value
		property var	mnmx		: [0.0, 500.0]		// Scale [min, max]
		property real	slen		: mnmx[1] - mnmx[0]	// Scale length
		property var    sme			: []				// min, average, max values
		property var    scale		: 1					// Scale is 1:1
		property var    stepsz		: 0.1				// stepSize (relative to Scale)
		property var    decplaces	: 4					// decimal places to display
		property var    suffix		: ''
	}
	Item {
		id:	fpid2										// Formal Parameter
		property string ptext		: 'Delay'			// Formal Parameter title shown on filter
		property real   dflt		: 0.0				// Default value
		property var	mnmx		: [-100.0, 2000.0]	// Scale [min, max]
		property real	slen		: mnmx[1] - mnmx[0]	// Scale length
		property var    sme			: []				// min, average, max values
		property var    scale		: 1					// Scale is 1:1
		property var    stepsz		: 0.1				// stepSize (relative to Scale)
		property var    decplaces	: 1					// decimal places to display
		property var    suffix      : ' s|%'
	}
	Item {
		id:	fpid3										// Formal Parameter
		property string ptext		: 'Duration'		// Formal Parameter title shown on filter
		property real   dflt		: -100.0				// Default value
		property var	mnmx		: [-100.0, 20.0]	// Scale [min, max]
		property real	slen		: mnmx[1] - mnmx[0]	// Scale length
		property var    sme			: []				// min, average, max values
		property var    scale		: 1					// Scale is 1:1
		property var    stepsz		: 0.1				// stepSize (relative to Scale)
		property var    decplaces	: 1					// decimal places to display
		property var    suffix      : ' s|%'
	}
	Item {
		id:	fpid4										// Formal Parameter
		property string ptext		: 'Angle'			// Formal Parameter title shown on filter
		property real   dflt		: 0.0				// Default value
		property var	mnmx		: [-60.0, 60.0]		// Scale [min, max]
		property real	slen		: mnmx[1] - mnmx[0]	// Scale length
		property var    sme			: []				// min, average, max values
		property var    scale		: 1					// Scale is 1:1
		property var    stepsz		: 1					// stepSize (relative to Scale)
		property var    decplaces	: 0					// decimal places to display
		property var    suffix      : ' °'
	}
	Item {
		id:	fpid5										// Formal Parameter
		property string ptext		: 'Direction'		// Formal Parameter title shown on filter
		property real   dflt		: 3					// Default "currentIndex" value
		property var    model		: ['top', 'right', 'bottom', 'left']
	}
	Item {
		id:	fpid6										// Formal Parameter
		property string ptext		: 'Main Color'		// Formal Parameter title shown on filter
		property string dflt		: '#add8e6'			// Default value
	}
	Item {
		id:	fpid7										// Formal Parameter
		property string ptext		: 'Stripe Color'	// Formal Parameter title shown on filter
		property string dflt		: '#5f4f97'			// Default value
	}
	Item {
		id:	fpid8										// Formal Parameter
		property string ptext		: 'Chroma'			// Formal Parameter title shown on filter
		property bool   dflt		: false				// Default value
	}
	Item {
		id:	fpid9										// Formal Parameter
		property string ptext		: 'Reverse'			// Formal Parameter title shown on filter
		property bool   dflt		: false				// Default value
	}
	Item {
		id:	fpid10										// Formal Parameter
		property string ptext		: 'Mask'			// Formal Parameter title shown on filter
		property bool   dflt		: false				// Default value
	}
	Item {
		id:	fpid11										// Formal Parameter
		property string ptext		: 'Chroma Color'	// Formal Parameter title shown on filter
		property string dflt		: '#00ff00'			// Default value
	}
	
    Component.onCompleted: {
        if (filter.isNew) {
 			filter.set('resource', filter.path + filterName + '.html')
		
            filter.set(fp0 , 'Shotcut filter: ' + filterName)
            filter.set(fp1 , fpid1 .dflt)
            filter.set(fp2 , fpid2 .dflt)
            filter.set(fp3 , fpid3 .dflt)
            filter.set(fp4 , fpid4 .dflt)
            filter.set(fp5 , fpid5 .dflt)
			filter.set(fp6 , fpid6 .dflt)
            filter.set(fp7 , fpid7 .dflt)
            filter.set(fp8 , fpid8 .dflt)
            filter.set(fp9 , fpid9 .dflt)
            filter.set(fp10, fpid10.dflt)
            filter.set(fp11, fpid11.dflt)
			filter.savePreset(preset.parameters)
		}
        setControls();
        filter.set('in' , producer.in)
        filter.set('out', producer.out)
    }

    function getPosition() {
        return Math.max(producer.position - (filter.in - producer.in), 0)
    }	
	
    function setControls() {		// Non-Key-framed
        var position     = getPosition()

        fpid1Slider     .value   = filter.getDouble(fp1 ) * fpid1.scale
		fpid2Slider     .value   = filter.getDouble(fp2 ) * fpid2.scale
        fpid3Slider     .value   = filter.getDouble(fp3 ) * fpid3.scale
        fpid4Slider     .value   = filter.getDouble(fp4 ) * fpid4.scale
        fpid5ComboBox   .currentIndex = filter.getDouble(fp5 )
        fpid6Swatch     .value   = filter.get      (fp6 )
        fpid7Swatch     .value   = filter.get      (fp7 )
        fpid8CheckBox   .checked = filter.getDouble(fp8 ) != 0.0
        fpid9CheckBox   .checked = filter.getDouble(fp9 ) != 0.0
        fpid10CheckBox  .checked = filter.getDouble(fp10) != 0.0
        fpid11Swatch    .value   = filter.get      (fp11)
    }

    GridLayout {
        columns			: 4
        anchors.fill	: parent
        anchors.margins	: 8

        Label {
            text: qsTr('Preset')
            Layout.alignment: Qt.AlignRight
        }

       Preset {
            id					: preset
            Layout.columnSpan	: parent.columns - 1  
			parameters			: [fp0, fp1, fp2, fp3, fp4, fp5, fp6, fp7, fp8, fp9, fp10, fp11]	
			onPresetSelected	: { setControls() }
        }


        Label {
            text				: qsTr(fpid1.ptext)		// Framerate (Non-Key-framed)
            Layout.alignment	: Qt.AlignRight
        }
        SliderSpinner {
            id					: fpid1Slider
			minimumValue		: fpid1.mnmx[0]
            maximumValue		: fpid1.mnmx[1]
			stepSize			: fpid1.stepsz
			decimals			: fpid1.decplaces
			suffix				: fpid1.suffix
            value				: filter.getDouble(fp1) * fpid1.scale
			onValueChanged		: filter.set      (fp1, value / fpid1.scale)
			Layout.columnSpan	: 2
        }
        UndoButton {
            onClicked			: fpid1Slider.value = fpid1.dflt * fpid1.scale
		}

		Label {
            text				: qsTr(fpid2.ptext)		// Delay
            Layout.alignment	: Qt.AlignRight
        }
        SliderSpinner {
            id					: fpid2Slider
			minimumValue		: fpid2.mnmx[0]
            maximumValue		: fpid2.mnmx[1]
			stepSize			: fpid2.stepsz
			decimals			: fpid2.decplaces
 			suffix				: fpid2.suffix
           value				: filter.getDouble(fp2) * fpid2.scale
            onValueChanged		: filter.set      (fp2, value / fpid2.scale)
			Layout.columnSpan	: 2
        }
        UndoButton {
            onClicked			: fpid2Slider.value = fpid2.dflt * fpid2.scale
		}

        Label {
            text				: qsTr(fpid3.ptext)		// Duration
            Layout.alignment	: Qt.AlignRight
        }
        SliderSpinner {
            id					: fpid3Slider
			minimumValue		: fpid3.mnmx[0]
            maximumValue		: fpid3.mnmx[1]
			stepSize			: fpid3.stepsz
			decimals			: fpid3.decplaces
 			suffix				: fpid3.suffix
           value				: filter.getDouble(fp3) * fpid3.scale
            onValueChanged		: filter.set      (fp3, value / fpid3.scale)
			Layout.columnSpan	: 2
        }
        UndoButton {
            onClicked			: fpid3Slider.value = fpid3.dflt * fpid3.scale
        }

        Label {
            text				: qsTr(fpid4.ptext)		// Angle
            Layout.alignment	: Qt.AlignRight
        }
        SliderSpinner {
            id					: fpid4Slider
			minimumValue		: fpid4.mnmx[0]
            maximumValue		: fpid4.mnmx[1]
			stepSize			: fpid4.stepsz
			decimals			: fpid4.decplaces
			suffix				: fpid4.suffix
            value				: filter.getDouble(fp4) * fpid4.scale
            onValueChanged		: filter.set      (fp4, value / fpid4.scale)
 			Layout.columnSpan	: 2
       }
        UndoButton {
            onClicked			: fpid4Slider.value = fpid4.dflt * fpid4.scale
        }
 
        Label {
            text					: qsTr(fpid5.ptext)	// Direction (0=top, 1=right, 2=bottom, 3=left)
            Layout.alignment		: Qt.AlignRight
        }
		ComboBox {
            id						: fpid5ComboBox
			model                   : fpid5.model
            Layout.alignment		: Qt.AlignLeft
            implicitWidth			: 100
			onCurrentIndexChanged	: filter.set(fp5, currentIndex)
        }
		Label { text				:'' }
        Item  { Layout.fillWidth	: true }
		
        Label {
            text					: qsTr(fpid6.ptext)		// Main Color
            Layout.alignment		: Qt.AlignRight
        }
        ColorPicker {
            id						: fpid6Swatch
            value					: filter.get(fp6)
            property bool isReady	: false
            Component.onCompleted	: isReady = true
            onValueChanged			: {
                if (isReady) {
                    filter.set(fp6, value)
                    filter.set("disable" , 0);
                }
            }
            onPickStarted			: filter.set("disable", 1)
            onPickCancelled			: filter.set('disable', 0)
        }
		Label { text				:'' }
		UndoButton {
            onClicked				: fpid6Swatch.value = fpid6.dflt
        }
		
        Label {
            text					: qsTr(fpid7.ptext)		// Stripe Color
            Layout.alignment		: Qt.AlignRight
        }
        ColorPicker {
            id						: fpid7Swatch
            value					: filter.get(fp7)
            property bool isReady	: false
            Component.onCompleted	: isReady = true
            onValueChanged			: {
                if (isReady) {
                    filter.set(fp7, value)
                    filter.set("disable" , 0);
                }
            }
            onPickStarted			: filter.set("disable", 1)
            onPickCancelled			: filter.set('disable', 0)
        }
 		Label { text				:'' }
		UndoButton {
            onClicked				: fpid7Swatch.value = fpid7.dflt
        }
 
        Label {
            text					: qsTr('Options')		// Chroma & Mask
            Layout.alignment		: Qt.AlignRight
        }
		CheckBox {
            id						: fpid8CheckBox
            Layout.alignment		: Qt.AlignLeft
            checked					: filter.getDouble(fp8)
            text					: qsTr(fpid8.ptext)
            onCheckedChanged		: filter.set(fp8, checked)
        }
 
		CheckBox {
            id						: fpid9CheckBox
            Layout.alignment		: Qt.AlignLeft
            checked					: filter.getDouble(fp9)
            text					: qsTr(fpid9.ptext)
            onCheckedChanged		: filter.set(fp9, checked)
        }
 
		CheckBox {
            id						: fpid10CheckBox
            Layout.alignment		: Qt.AlignLeft
            checked					: filter.getDouble(fp10)
            text					: qsTr(fpid10.ptext)
            onCheckedChanged		: filter.set(fp10, checked)
        }
		
        Label {
            text					: qsTr(fpid11.ptext)		// Chroma Color
            Layout.alignment		: Qt.AlignRight
        }
        ColorPicker {
            id						: fpid11Swatch
            value					: filter.get(fp10)
            property bool isReady	: false
            Component.onCompleted	: isReady = true
            onValueChanged			: {
                if (isReady) {
                    filter.set(fp11, value)
                    filter.set("disable" , 0);
                }
            }
            onPickStarted			: filter.set("disable", 1)
            onPickCancelled			: filter.set('disable', 0)
        }
 		Label { text				:'' }
        UndoButton {
            onClicked				: fpid11Swatch.value = fpid11.dflt
        }
		
        Label {
            text					: qsTr('Description')		// Link to filter description
			Layout.alignment		: Qt.AlignRight
       }
		Button {
            id						: description
            Layout.alignment		: Qt.AlignLeft
			text					: "Details"
			onClicked				: Qt.openUrlExternally('http://elusien.co.uk/shotcut/filters/' + filterName)
        }		
        Item { Layout.fillWidth		: true }
	}

}
